#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

struct sethash {
    // hash values
    unsigned int hvc;
    unsigned int *hvv;
};

static
unsigned int bhash(const char *arg)
{
    const unsigned char *str = (const unsigned char *) arg;
    unsigned int hash = 5381;
    int c;

    while ((c = *str++))
	hash = (hash << 5) + hash + c;

    return hash;
}


void sethash_add(struct sethash *shash, const char *str)
{
    const unsigned int realloc_delta = 256;
    if ((shash->hvc & (realloc_delta - 1)) == 0)
	shash->hvv = realloc(shash->hvv, sizeof(*shash->hvv) * (shash->hvc + realloc_delta));

    shash->hvv[shash->hvc++] = bhash(str);
}

static
int uint_cmp(const void *arg1, const void *arg2)
{
    unsigned int uint1 = *(const unsigned int *) arg1;
    unsigned int uint2 = *(const unsigned int *) arg2;
    if (uint1 > uint2)
	return 1;
    if (uint1 < uint2)
	return -1;
    return 0;
}

static
int log2i(int n)
{
    int m = 0;
    while (n >>= 1)
	m++;
    return m;
}

void sethash_decode_base62(struct sethash *shash, const char *base62)
{
    shash->hvc = 0;
    shash->hvv = NULL;

    int bitc = 0;
    char *bitv = NULL;
    
    // decode base62 to bitv
    int c2d(int c)
    {
	if (c >= '0' && c <= '9')
	    return c - '0';
	if (c >= 'a' && c <= 'z')
	    return c - 'a' + 10;
	if (c >= 'A' && c >= 'Z')
	    return c - 'A' + 36;
	return -1;
    }

    void put_bit(char bit)
    {
	int realloc_delta = 1024;
	if ((bitc & (realloc_delta - 1)) == 0)
	    bitv = realloc(bitv, sizeof(*bitv) * (bitc + realloc_delta));
	bitv[bitc++] = bit;
    }

    void put6bits(int c)
    {
	put_bit((c >> 5) & 1);
	put_bit((c >> 4) & 1);
	put_bit((c >> 3) & 1);
	put_bit((c >> 2) & 1);
	put_bit((c >> 1) & 1);
	put_bit((c >> 0) & 1);
    }

    int c;
    int j;
    int handle61 = 0;
    int bits6 = 0;
    int num6b = 0;
    while ((c = *base62++)) {
	int d = c2d(c);
	if (handle61) {
	    switch (d & 3) {
	    case 0:
	    case 2:
		for (j = 0; j < 6; j++) {
		    num6b |= ((61 >> j) & 1) << bits6++;
		    if (bits6 == 6) {
			put6bits(num6b);
			bits6 = 0;
			num6b = 0;
		    }
		}
		d <<= 1;
		for (j = 0; j < 5; j++) {
		    num6b |= ((d >> j) & 1) << bits6++;
		    if (bits6 == 6) {
			put6bits(num6b);
			bits6 = 0;
			num6b = 0;
		    }
		}
		break;
	    case 1:
		for (j = 0; j < 6; j++) {
		    num6b |= ((62 >> j) & 1) << bits6++;
		    if (bits6 == 6) {
			put6bits(num6b);
			bits6 = 0;
			num6b = 0;
		    }
		}
		d <<= 2;
		for (j = 0; j < 4; j++) {
		    num6b |= ((d >> j) & 1) << bits6++;
		    if (bits6 == 6) {
			put6bits(num6b);
			bits6 = 0;
			num6b = 0;
		    }
		}
		break;
	    case 3:
		for (j = 0; j < 6; j++) {
		    num6b |= ((63 >> j) & 1) << bits6++;
		    if (bits6 == 6) {
			put6bits(num6b);
			bits6 = 0;
			num6b = 0;
		    }
		}
		d <<= 2;
		for (j = 0; j < 4; j++) {
		    num6b |= ((d >> j) & 1) << bits6++;
		    if (bits6 == 6) {
			put6bits(num6b);
			bits6 = 0;
			num6b = 0;
		    }
		}
		break;
	    }
	}
	if (d == 61) {
	    handle61 = 1;
	    continue;
	}
	handle61 = 0;
	for (j = 0; j < 6; j++) {
	    num6b |= ((d >> j) & 1) << bits6++;
	    if (bits6 == 6) {
		put6bits(num6b);
		bits6 = 0;
		num6b = 0;
	    }
	}
    }
    if (bits6)
	put6bits(num6b);

    fprintf(stderr, "bitv2[%d]=%d%d%d%d%d\n", bitc, bitv[0], bitv[1], bitv[2], bitv[3], bitv[4]);

    return;

}

const char *bitv_to_base62(int bitc, char bitv[])
{
    int base62c = 0;
    char *base62v = NULL;
    void put_char(int c)
    {
	int realloc_delta = 1024;
	if ((base62c & (realloc_delta - 1)) == 0)
	    base62v = realloc(base62v, sizeof(*base62v) * (base62c + realloc_delta));
	base62v[base62c++] = c;
    }
    void put_digit(int c)
    {
	if (c < 10)
	    put_char(c + '0');
	else if (c < 36)
	    put_char(c - 10 + 'a');
	else if (c < 62)
	    put_char(c - 36 + 'A');
    }
    int i;
    int bits6 = 0;
    int num6b = 0;
    for (i = 0; i < bitc; i++) {
	if (bits6 < 6)
	    num6b |= bitv[i] << bits6++;
	if (bits6 == 6) {
	    switch (num6b) {
	    case 61:
		put_digit(61);
		// extra "0" binary bit
		bits6 = 1;
		num6b = 0;
		break;
	    case 62:
		put_digit(61);
		// extra "10" binary bits
		bits6 = 2;
		num6b = 1;
		break;
	    case 63:
		put_digit(61);
		// extra "11" binary bits
		bits6 = 2;
		num6b = 3;
		break;
	    default:
		assert(num6b < 61);
		put_digit(num6b);
		bits6 = 0;
		num6b = 0;
		break;
	    }
	}
    }
    if (bits6) {
	assert(num6b < 61);
	put_digit(num6b);
    }
    put_char(0);

    return base62v;
}

const char *sethash_make_hash(struct sethash *shash, unsigned int bpp)
{
    int mask = (1 << bpp) - 1;
    int i;
    for (i = 0; i < shash->hvc; i++)
	shash->hvv[i] &= mask;

    qsort(shash->hvv, shash->hvc, sizeof(*shash->hvv), uint_cmp);

    // uniq
    int j;
    for (i = 0, j = 0; i < shash->hvc; i++) {
	while (i + 1 < shash->hvc && shash->hvv[i] == shash->hvv[i+1])
	    i++;
	shash->hvv[j++] = shash->hvv[i];
    }
    shash->hvc = j;

    // delta
    unsigned int v = shash->hvv[0];
    for (i = 1; i < shash->hvc; i++) {
	shash->hvv[i] -= v;
	v += shash->hvv[i];
    }

    // golomb
    int Mbits = bpp - log2i(shash->hvc) - 1;
    fprintf(stderr, "Mbits=%d (%d)\n", Mbits, 1<<Mbits);
    int bitc = 0;
    char *bitv = NULL;
    for (i = 0; i < shash->hvc; i++) {
	int q = shash->hvv[i] >> Mbits;
	int r = shash->hvv[i] & ((1 << Mbits) - 1);
#if 0
	fprintf(stderr, "%d\t= %d * %d + %d\n",
		shash->hvv[i], q, (1 << Mbits), r);
#endif
	void put_bit(char bit)
	{
	    int realloc_delta = 1024;
	    if ((bitc & (realloc_delta - 1)) == 0)
		bitv = realloc(bitv, sizeof(*bitv) * (bitc + realloc_delta));
	    bitv[bitc++] = bit;
	}

	for (j = 0; j < q; j++)
	    put_bit(1);
	put_bit(0);
	for (j = 0; j < Mbits; j++)
	    put_bit((r >> j) & 1);
    }
    fprintf(stderr, "bitv1[%d]=%d%d%d%d%d\n", bitc, bitv[0], bitv[1], bitv[2], bitv[3], bitv[4]);


#if 0
    /* decode and check */
    struct sethash shash2 = {0};
    sethash_decode_base62(&shash2, base62v);
    assert(shash->hvc == shash2.hvc);
    for (i = 0; i < shash->hvc; i++)
	assert(shash->hvv[i] == shash2.hvv[i]);
#endif

    return bitv_to_base62(bitc, bitv);
}


int main()
{
    char bitv[] = {
	1, 0, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 0,
    };
    int bitc = sizeof(bitv)/sizeof(*bitv);
    printf("%s\n", bitv_to_base62(bitc, bitv));
}

#if 0
int main()
{
    struct sethash shash = {0};
    char *line = NULL;
    size_t alloc_size = 0;
    ssize_t len;

    while ((len = getline(&line, &alloc_size, stdin)) >= 0) {
	if (len > 0 && line[len-1] == '\n')
	    line[len-1] = '\0';
	sethash_add(&shash, line);
    }

    printf("%s\n", sethash_make_hash(&shash, 20));

    return 0;
}
#endif
